
require 'racc/parser'
=begin
= rdinlineparser.ry
== What is rdinlineparser.ry
rdinlineparser.ry is Racc syntax definition file for Inline level
parsing. rdinlineparser.tab.rb is required by rdblockparser.tab.rb,
and RDInlineParser is used RDBlockParser.

Please read rdblockparser.ry for more details.

== Inline level parsing
Inline level parsing is very simple. You can easily understand it
when you read BNF in rdinlineparser.ry.

Now RDInlineParser can parse only regular RD. "regular" means Inline
brackets are exactly nested. For example,
  This is regular
  ((- Footnote and ((*Em*)). -))
  And they are ilregular
  ((- Footnote and ((*Emphasis -)) . *))
  ((- Not closed pair of bracket.
When RDInlineParser meats such a ilregular RD, RDInlineParser raises
a Exception...
== Inline level lexical analizing
In Inline level, lexer do all lexical analizing before parsing. Each
tokens are pushed into (({@token_queue})) and each values into 
(({@value_queue})).
=end
=begin
== script info.
 parser for RD Inline.
 $Id: rdinlineparser.ry,v 1.26.4.1 2000/09/21 17:02:34 toshirok Exp $

=end

require "strscan"

module RD
class RDInlineParser < Racc::Parser
EM_OPEN = '((*'
EM_OPEN_RE = /#{Regexp.quote(EM_OPEN)}/
EM_CLOSE = '*))'
EM_CLOSE_RE = /#{Regexp.quote(EM_CLOSE)}/
CODE_OPEN = '(({'
CODE_OPEN_RE = /#{Regexp.quote(CODE_OPEN)}/
CODE_CLOSE = '}))'
CODE_CLOSE_RE = /#{Regexp.quote(CODE_CLOSE)}/
VAR_OPEN = '((|'
VAR_OPEN_RE = /#{Regexp.quote(VAR_OPEN)}/
VAR_CLOSE = '|))'
VAR_CLOSE_RE = /#{Regexp.quote(VAR_CLOSE)}/
KBD_OPEN = '((%'
KBD_OPEN_RE = /#{Regexp.quote(KBD_OPEN)}/
KBD_CLOSE = '%))'
KBD_CLOSE_RE = /#{Regexp.quote(KBD_CLOSE)}/
INDEX_OPEN = '((:'
INDEX_OPEN_RE = /#{Regexp.quote(INDEX_OPEN)}/
INDEX_CLOSE = ':))'
INDEX_CLOSE_RE = /#{Regexp.quote(INDEX_CLOSE)}/
REF_OPEN = '((<'
REF_OPEN_RE = /#{Regexp.quote(REF_OPEN)}/
REF_CLOSE = '>))'
REF_CLOSE_RE = /#{Regexp.quote(REF_CLOSE)}/
FOOTNOTE_OPEN = '((-'
FOOTNOTE_OPEN_RE = /#{Regexp.quote(FOOTNOTE_OPEN)}/
FOOTNOTE_CLOSE = '-))'
FOOTNOTE_CLOSE_RE = /#{Regexp.quote(FOOTNOTE_CLOSE)}/
VERB_OPEN = "(('"
VERB_OPEN_RE = /#{Regexp.quote(VERB_OPEN)}/
VERB_CLOSE = "'))"
VERB_CLOSE_RE = /#{Regexp.quote(VERB_CLOSE)}/

BAR = "|"
BAR_RE = /#{Regexp.quote(BAR)}/
QUOTE = '"'
QUOTE_RE = /#{Regexp.quote(QUOTE)}/
SLASH = "/"
SLASH_RE = /#{Regexp.quote(SLASH)}/
BACK_SLASH = "\\"
BACK_SLASH_RE = /#{Regexp.quote(BACK_SLASH)}/
URL = "URL:"
URL_RE = /#{Regexp.quote(URL)}/

# Workaround for Regexp option change of Ruby 1.5.x
other_re_mode = Regexp::EXTENDED
if RUBY_VERSION > "1.5"
  other_re_mode |= Regexp::MULTILINE
else
  other_re_mode |= Regexp::POSIXLINE
end

OTHER_RE = Regexp.new(
		  ".+?(?=#{Regexp.quote(EM_OPEN)}|#{Regexp.quote(EM_CLOSE)}|
                  #{Regexp.quote(CODE_OPEN)}|#{Regexp.quote(CODE_CLOSE)}|
                  #{Regexp.quote(VAR_OPEN)}|#{Regexp.quote(VAR_CLOSE)}|
                  #{Regexp.quote(KBD_OPEN)}|#{Regexp.quote(KBD_CLOSE)}|
                  #{Regexp.quote(INDEX_OPEN)}|#{Regexp.quote(INDEX_CLOSE)}|
                  #{Regexp.quote(REF_OPEN)}|#{Regexp.quote(REF_CLOSE)}|
                #{Regexp.quote(FOOTNOTE_OPEN)}|#{Regexp.quote(FOOTNOTE_CLOSE)}|
                  #{Regexp.quote(VERB_OPEN)}|#{Regexp.quote(VERB_CLOSE)}|
                  #{Regexp.quote(BAR)}|
                  #{Regexp.quote(QUOTE)}|
                  #{Regexp.quote(SLASH)}|
                  #{Regexp.quote(BACK_SLASH)}|
                  #{Regexp.quote(URL)})", other_re_mode)

def initialize(bp)
  @blockp = bp
end

def parse(src)
  @src = StringScanner.new(src)
  @pre = ""
  @yydebug = true
  do_parse
end

def next_token
  return [false, false] if @src.empty?
#  p @src.rest if @yydebug
  if ret = @src.scan(EM_OPEN_RE)
    @pre << ret
    [:EM_OPEN, ret]
  elsif ret = @src.scan(EM_CLOSE_RE)
    @pre << ret
    [:EM_CLOSE, ret]
  elsif ret = @src.scan(CODE_OPEN_RE)
    @pre << ret
    [:CODE_OPEN, ret]
  elsif ret = @src.scan(CODE_CLOSE_RE)
    @pre << ret
    [:CODE_CLOSE, ret]
  elsif ret = @src.scan(VAR_OPEN_RE)
    @pre << ret
    [:VAR_OPEN, ret]
  elsif ret = @src.scan(VAR_CLOSE_RE)
    @pre << ret
    [:VAR_CLOSE, ret]
  elsif ret = @src.scan(KBD_OPEN_RE)
    @pre << ret
    [:KBD_OPEN, ret]
  elsif ret = @src.scan(KBD_CLOSE_RE)
    @pre << ret
    [:KBD_CLOSE, ret]
  elsif ret = @src.scan(INDEX_OPEN_RE)
    @pre << ret
    [:INDEX_OPEN, ret]
  elsif ret = @src.scan(INDEX_CLOSE_RE)
    @pre << ret
    [:INDEX_CLOSE, ret]
  elsif ret = @src.scan(REF_OPEN_RE)
    @pre << ret
    [:REF_OPEN, ret]
  elsif ret = @src.scan(REF_CLOSE_RE)
    @pre << ret
    [:REF_CLOSE, ret]
  elsif ret = @src.scan(FOOTNOTE_OPEN_RE)
    @pre << ret
    [:FOOTNOTE_OPEN, ret]
  elsif ret = @src.scan(FOOTNOTE_CLOSE_RE)
    @pre << ret
    [:FOOTNOTE_CLOSE, ret]
  elsif ret = @src.scan(VERB_OPEN_RE)
    @pre << ret
    [:VERB_OPEN, ret]
  elsif ret = @src.scan(VERB_CLOSE_RE)
    @pre << ret
    [:VERB_CLOSE, ret]
  elsif ret = @src.scan(BAR_RE)
    @pre << ret
    [:BAR, ret]
  elsif ret = @src.scan(QUOTE_RE)
    @pre << ret
    [:QUOTE, ret]
  elsif ret = @src.scan(SLASH_RE)
    @pre << ret
    [:SLASH, ret]
  elsif ret = @src.scan(BACK_SLASH_RE)
    @pre << ret
    [:BACK_SLASH, ret]
  elsif ret = @src.scan(URL_RE)
    @pre << ret
    [:URL, ret]
  elsif ret = @src.scan(OTHER_RE)
    @pre << ret
    [:OTHER, ret]
  else
    ret = @src.rest
    @pre << ret
    @src.clear
    [:OTHER, ret]
  end
end

def on_error(et, ev, values)
  line_num = @pre.to_a.size - 1
  if /\n/ =~ @src.rest
    next_words = $`
  else
    next_words = @src.rest
  end
  
  raise ParseError, <<Msg

RD syntax error: line #{@blockp.line_index - line_num}:
... #{(ev||'') + next_words} ...
   ^
Msg
end

##### racc 1.2.4 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 1, 27, :_reduce_none,
 2, 28, :_reduce_2,
 1, 28, :_reduce_3,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 1, 29, :_reduce_none,
 3, 30, :_reduce_13,
 3, 31, :_reduce_14,
 3, 32, :_reduce_15,
 3, 33, :_reduce_16,
 3, 34, :_reduce_17,
 4, 35, :_reduce_18,
 3, 35, :_reduce_19,
 2, 40, :_reduce_20,
 2, 40, :_reduce_21,
 1, 40, :_reduce_22,
 1, 40, :_reduce_23,
 2, 41, :_reduce_24,
 2, 41, :_reduce_25,
 1, 41, :_reduce_26,
 1, 41, :_reduce_27,
 2, 39, :_reduce_none,
 4, 39, :_reduce_29,
 4, 39, :_reduce_30,
 2, 43, :_reduce_none,
 4, 43, :_reduce_32,
 1, 44, :_reduce_33,
 3, 44, :_reduce_34,
 1, 45, :_reduce_none,
 3, 45, :_reduce_36,
 3, 45, :_reduce_37,
 2, 46, :_reduce_38,
 2, 46, :_reduce_39,
 1, 46, :_reduce_40,
 1, 46, :_reduce_41,
 1, 47, :_reduce_none,
 2, 51, :_reduce_43,
 1, 51, :_reduce_44,
 2, 53, :_reduce_45,
 1, 53, :_reduce_46,
 1, 50, :_reduce_none,
 1, 50, :_reduce_none,
 1, 50, :_reduce_none,
 1, 50, :_reduce_none,
 1, 50, :_reduce_none,
 1, 50, :_reduce_none,
 1, 54, :_reduce_none,
 1, 54, :_reduce_none,
 1, 55, :_reduce_none,
 1, 55, :_reduce_none,
 1, 56, :_reduce_57,
 1, 52, :_reduce_58,
 1, 57, :_reduce_59,
 2, 58, :_reduce_60,
 1, 58, :_reduce_none,
 2, 49, :_reduce_62,
 1, 49, :_reduce_none,
 2, 48, :_reduce_64,
 1, 48, :_reduce_none,
 1, 60, :_reduce_none,
 1, 60, :_reduce_none,
 1, 60, :_reduce_none,
 1, 60, :_reduce_none,
 1, 60, :_reduce_none,
 1, 62, :_reduce_none,
 1, 62, :_reduce_none,
 1, 59, :_reduce_none,
 1, 59, :_reduce_none,
 1, 61, :_reduce_none,
 1, 61, :_reduce_none,
 1, 61, :_reduce_none,
 2, 42, :_reduce_78,
 1, 42, :_reduce_none,
 1, 63, :_reduce_none,
 2, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 1, 63, :_reduce_none,
 3, 36, :_reduce_101,
 3, 37, :_reduce_102,
 1, 65, :_reduce_none,
 1, 65, :_reduce_none,
 1, 65, :_reduce_none,
 1, 65, :_reduce_none,
 1, 65, :_reduce_none,
 1, 65, :_reduce_none,
 2, 66, :_reduce_109,
 1, 66, :_reduce_none,
 1, 38, :_reduce_111,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 1, 67, :_reduce_none,
 2, 67, :_reduce_131,
 2, 67, :_reduce_132,
 1, 67, :_reduce_none,
 2, 64, :_reduce_134,
 1, 64, :_reduce_none ]

racc_reduce_n = 136

racc_shift_n = 177

racc_action_table = [
   127,   129,   132,   135,   137,   118,   120,   122,   125,   115,
   131,   134,   136,   117,   121,   123,   126,   128,   130,   133,
   145,   116,   119,   127,   129,   132,   135,   137,   118,   120,
   122,   125,   162,   131,   134,   136,   117,   121,   123,   126,
   128,   130,   133,   163,   116,   119,   127,   129,   132,   135,
   137,   118,   120,   122,   125,    87,   131,   134,   136,   117,
   121,   123,   126,   128,   130,   133,   151,   116,   119,    40,
    42,    45,    46,    49,    31,    33,    35,    38,   146,    44,
    48,    51,    30,    34,    36,    39,    41,    43,    47,    50,
    29,    83,   127,   129,   132,   135,   137,   118,   120,   122,
   125,    88,   131,   134,   136,   117,   121,   123,   126,   128,
   130,   133,   147,   116,   119,    11,    15,    18,    23,    26,
     1,     5,     9,   169,   170,    20,    86,    27,   158,     6,
   171,    12,    87,    19,   173,    78,     2,    11,    15,    18,
    23,    26,     1,     5,     9,    85,   149,    20,   150,    27,
   170,     6,   148,    12,   nil,    19,    92,   nil,     2,    68,
    98,    52,    54,    58,    61,   nil,    20,   nil,    27,   nil,
     6,   nil,    12,   nil,    19,   nil,   nil,     2,    11,    15,
    18,    23,    26,     1,     5,     9,   nil,   nil,    20,   nil,
    27,   nil,     6,   nil,    12,   nil,    19,   nil,   nil,     2,
    11,    15,    18,    23,    26,     1,     5,     9,   nil,   nil,
    20,   nil,    27,   nil,     6,   nil,    12,   nil,    19,   nil,
   nil,     2,    11,    15,    18,    23,    26,     1,     5,     9,
   nil,   nil,    20,   nil,    27,   nil,     6,   nil,    12,   nil,
    19,   nil,   nil,     2,    11,    15,    18,    23,    26,     1,
     5,     9,   nil,   nil,    20,   nil,    27,   nil,     6,   nil,
    12,   nil,    19,   nil,   nil,     2,    11,    15,    18,    23,
    26,     1,     5,     9,   nil,   nil,    20,   nil,    27,   nil,
     6,   nil,    12,   nil,    19,    92,   nil,     2,    68,    98,
    52,    54,    58,    61,   nil,    20,   nil,    27,   nil,     6,
   nil,    12,   nil,    19,   nil,   nil,     2,    40,    42,    45,
    46,    49,    31,    33,    35,    38,   nil,    44,    48,    51,
    30,    34,    36,    39,    41,    43,    47,    50,    29,    11,
    15,    18,    23,    26,     1,     5,     9,   nil,   nil,    20,
   nil,    27,   nil,     6,   nil,    12,   nil,    19,    92,   nil,
     2,    68,    98,    52,    54,    58,    61,   nil,    20,   nil,
    27,   nil,     6,   nil,    12,   nil,    19,    62,   nil,     2,
    68,    72,    52,    54,    58,    61,   nil,    20,   nil,    27,
   nil,     6,   nil,    12,   nil,    19,    92,   nil,     2,    68,
    98,    52,    54,    58,    61,   nil,    20,   nil,    27,   nil,
     6,   nil,    12,   110,    19,   nil,    68,     2,    52,    54,
    58,    61,   nil,    20,   nil,    27,   nil,     6,   nil,    12,
   nil,    19,   nil,   nil,     2,   102,   103,    68,    98,    52,
    54,    58,    61,   nil,    20,   nil,    27,   nil,     6,   nil,
    12,   nil,    19,   nil,   nil,     2,   102,   103,    68,    98,
    52,    54,    58,    61,   nil,    20,   nil,    27,   nil,     6,
   nil,    12,   nil,    19,   nil,   nil,     2,   102,   103,    68,
    98,    52,    54,    58,    61,   nil,    20,   nil,    27,   nil,
     6,   nil,    12,   nil,    19,    92,   nil,     2,    68,    98,
    52,    54,    58,    61,   172,   102,   103,    68,    98,    52,
    54,    58,    61,   141,   nil,   nil,    68,   144,    52,    54,
    58,    61,    92,   nil,   nil,    68,    98,    52,    54,    58,
    61,   166,   nil,   nil,    68,   nil,    52,    54,    58,    61,
   154,   102,   103,    68,    98,    52,    54,    58,    61,   174,
   102,   103,    68,    98,    52,    54,    58,    61,   176,   102,
   103,    68,    98,    52,    54,    58,    61,   102,   103,    68,
    98,    52,    54,    58,    61,   102,   103,    68,    98,    52,
    54,    58,    61,   102,   103,    68,    98,    52,    54,    58,
    61,    11,    15,    18,    23,    26,     1 ]

racc_action_check = [
   144,   144,   144,   144,   144,   144,   144,   144,   144,    70,
   144,   144,   144,   144,   144,   144,   144,   144,   144,   144,
    74,   144,   144,   124,   124,   124,   124,   124,   124,   124,
   124,   124,   135,   124,   124,   124,   124,   124,   124,   124,
   124,   124,   124,   139,   124,   124,    72,    72,    72,    72,
    72,    72,    72,    72,    72,   140,    72,    72,    72,    72,
    72,    72,    72,    72,    72,    72,    82,    72,    72,    32,
    32,    32,    32,    32,    32,    32,    32,    32,    76,    32,
    32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
    32,    32,   168,   168,   168,   168,   168,   168,   168,   168,
   168,    57,   168,   168,   168,   168,   168,   168,   168,   168,
   168,   168,    77,   168,   168,     0,     0,     0,     0,     0,
     0,     0,     0,   154,   154,     0,    46,     0,   108,     0,
   158,     0,    56,     0,   160,    14,     0,    12,    12,    12,
    12,    12,    12,    12,    12,    46,    80,    12,    81,    12,
   174,    12,    78,    12,   nil,    12,    94,   nil,    12,    94,
    94,    94,    94,    94,    94,   nil,    94,   nil,    94,   nil,
    94,   nil,    94,   nil,    94,   nil,   nil,    94,    17,    17,
    17,    17,    17,    17,    17,    17,   nil,   nil,    17,   nil,
    17,   nil,    17,   nil,    17,   nil,    17,   nil,   nil,    17,
    19,    19,    19,    19,    19,    19,    19,    19,   nil,   nil,
    19,   nil,    19,   nil,    19,   nil,    19,   nil,    19,   nil,
   nil,    19,    20,    20,    20,    20,    20,    20,    20,    20,
   nil,   nil,    20,   nil,    20,   nil,    20,   nil,    20,   nil,
    20,   nil,   nil,    20,    27,    27,    27,    27,    27,    27,
    27,    27,   nil,   nil,    27,   nil,    27,   nil,    27,   nil,
    27,   nil,    27,   nil,   nil,    27,     6,     6,     6,     6,
     6,     6,     6,     6,   nil,   nil,     6,   nil,     6,   nil,
     6,   nil,     6,   nil,     6,   114,   nil,     6,   114,   114,
   114,   114,   114,   114,   nil,   114,   nil,   114,   nil,   114,
   nil,   114,   nil,   114,   nil,   nil,   114,     2,     2,     2,
     2,     2,     2,     2,     2,     2,   nil,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     9,
     9,     9,     9,     9,     9,     9,     9,   nil,   nil,     9,
   nil,     9,   nil,     9,   nil,     9,   nil,     9,    60,   nil,
     9,    60,    60,    60,    60,    60,    60,   nil,    60,   nil,
    60,   nil,    60,   nil,    60,   nil,    60,     5,   nil,    60,
     5,     5,     5,     5,     5,     5,   nil,     5,   nil,     5,
   nil,     5,   nil,     5,   nil,     5,    65,   nil,     5,    65,
    65,    65,    65,    65,    65,   nil,    65,   nil,    65,   nil,
    65,   nil,    65,    63,    65,   nil,    63,    65,    63,    63,
    63,    63,   nil,    63,   nil,    63,   nil,    63,   nil,    63,
   nil,    63,   nil,   nil,    63,   110,   110,   110,   110,   110,
   110,   110,   110,   nil,   110,   nil,   110,   nil,   110,   nil,
   110,   nil,   110,   nil,   nil,   110,   105,   105,   105,   105,
   105,   105,   105,   105,   nil,   105,   nil,   105,   nil,   105,
   nil,   105,   nil,   105,   nil,   nil,   105,    62,    62,    62,
    62,    62,    62,    62,    62,   nil,    62,   nil,    62,   nil,
    62,   nil,    62,   nil,    62,    64,   nil,    62,    64,    64,
    64,    64,    64,    64,   159,   159,   159,   159,   159,   159,
   159,   159,   159,    73,   nil,   nil,    73,    73,    73,    73,
    73,    73,    90,   nil,   nil,    90,    90,    90,    90,    90,
    90,   142,   nil,   nil,   142,   nil,   142,   142,   142,   142,
    99,    99,    99,    99,    99,    99,    99,    99,    99,   164,
   164,   164,   164,   164,   164,   164,   164,   164,   175,   175,
   175,   175,   175,   175,   175,   175,   175,   141,   141,   141,
   141,   141,   141,   141,   141,   156,   156,   156,   156,   156,
   156,   156,   156,   166,   166,   166,   166,   166,   166,   166,
   166,     8,     8,     8,     8,     8,     8 ]

racc_action_pointer = [
   112,   nil,   304,   nil,   nil,   364,   263,   nil,   578,   326,
   nil,   nil,   134,   nil,   135,   nil,   nil,   175,   nil,   197,
   219,   nil,   nil,   nil,   nil,   nil,   nil,   241,   nil,   nil,
   nil,   nil,    66,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   120,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   127,    78,   nil,   nil,
   345,   nil,   463,   400,   482,   383,   nil,   nil,   nil,   nil,
     5,   nil,    43,   500,     2,   nil,    67,    92,   152,   nil,
   124,   134,    50,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
   509,   nil,   nil,   nil,   153,   nil,   nil,   nil,   nil,   527,
   nil,   nil,   nil,   nil,   nil,   442,   nil,   nil,   125,   nil,
   421,   nil,   nil,   nil,   282,   nil,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,    20,   nil,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   nil,    26,   nil,   nil,   nil,    20,
    50,   553,   518,   nil,    -3,   nil,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,   119,   nil,   561,   nil,   126,   491,
   131,   nil,   nil,   nil,   536,   nil,   569,   nil,    89,   nil,
   nil,   nil,   nil,   nil,   145,   545,   nil ]

racc_action_default = [
  -136,  -103,  -136,    -6,  -110,  -136,  -136,    -7,  -111,  -136,
    -8,  -104,  -136,    -9,  -136,  -105,   -10,    -1,  -106,  -136,
  -136,   -11,    -3,  -107,   -12,    -4,  -108,  -136,    -5,  -130,
  -119,  -112,  -136,  -126,  -120,  -128,  -121,  -135,  -129,  -122,
  -113,  -123,  -114,  -124,  -116,  -115,  -136,  -125,  -117,  -133,
  -127,  -118,   -66,   -49,   -68,   -50,   -58,  -136,   -69,   -51,
   -41,   -70,  -136,   -27,   -63,   -40,   -52,   -26,   -67,   -47,
   -35,   -48,  -136,  -136,  -136,  -109,  -136,  -136,  -136,    -2,
  -136,  -136,  -136,  -102,  -134,  -131,  -132,   -31,   -19,   -54,
   -57,   -53,   -74,   -61,   -38,   -71,   -73,   -44,   -72,   -59,
   -56,   -55,   -76,   -77,   -65,   -42,   -75,   -46,  -136,   -58,
  -136,   -25,   -35,   -62,   -39,   -28,   -99,   -89,   -80,  -100,
   -96,   -90,   -97,   -91,   -24,   -98,   -92,   -85,   -93,   -84,
   -94,   -86,   -83,   -95,   -87,  -136,   -88,   -82,   -79,  -136,
   -33,  -136,   -23,   -22,  -136,   -15,  -101,   -16,   177,   -17,
   -13,   -14,   -60,   -43,   -37,   -64,   -59,   -45,   -36,   -59,
  -136,   -78,   -81,   -18,  -136,   -33,  -136,   -21,   -20,   -30,
   -32,   -29,   -37,   -36,   -34,  -136,   -34 ]

racc_goto_table = [
    56,    64,    99,    60,   124,   143,   155,    63,   108,    67,
   161,    37,    70,   153,   113,   152,    14,   157,    73,    96,
    25,   139,    74,    96,    96,    76,    25,    79,    77,    25,
    32,    75,    25,   153,    57,    80,    81,    25,   152,    25,
    25,    84,   nil,    82,   nil,   156,    94,    25,   nil,    96,
   159,   114,   nil,    96,   161,   nil,   160,   nil,   109,    64,
   101,    60,   nil,   155,   nil,   nil,   155,   111,   140,    64,
   112,   155,    96,    96,   167,   142,   168,    28,   nil,   nil,
   nil,   164,   155,    28,   nil,   nil,    28,   nil,   nil,    28,
   nil,   nil,   nil,    21,    28,   nil,    28,    28,   nil,    21,
   nil,   nil,    21,   101,    28,    21,   175,   nil,   101,    10,
    21,   nil,    21,    21,   nil,    10,   nil,   nil,    10,   nil,
    21,    10,   nil,   nil,   nil,   nil,    10,   nil,    10,    10,
   nil,   nil,   nil,     3,   nil,   nil,    10,   165,    64,     3,
   nil,   nil,     3,   nil,   nil,     3,   nil,   nil,   nil,     7,
     3,   nil,     3,     3,   nil,     7,   nil,   nil,     7,   nil,
     3,     7,   nil,   nil,   nil,   nil,     7,   nil,     7,     7,
   nil,   nil,   nil,   nil,   nil,   nil,     7 ]

racc_goto_check = [
    23,    34,    22,    24,    16,    18,    35,    17,    21,    19,
    37,    41,    20,    28,    32,    33,     1,    29,    13,    36,
     4,    14,     1,    36,    36,     1,     4,     3,     1,     4,
    38,    39,     4,    28,    15,     1,     1,     4,    33,     4,
     4,    41,   nil,     1,   nil,    22,    25,     4,   nil,    36,
    22,    25,   nil,    36,    37,   nil,    21,   nil,    23,    34,
    24,    24,   nil,    35,   nil,   nil,    35,    19,    23,    34,
    20,    35,    36,    36,    18,    17,    16,     5,   nil,   nil,
   nil,    22,    35,     5,   nil,   nil,     5,   nil,   nil,     5,
   nil,   nil,   nil,    11,     5,   nil,     5,     5,   nil,    11,
   nil,   nil,    11,    24,     5,    11,    22,   nil,    24,     8,
    11,   nil,    11,    11,   nil,     8,   nil,   nil,     8,   nil,
    11,     8,   nil,   nil,   nil,   nil,     8,   nil,     8,     8,
   nil,   nil,   nil,     6,   nil,   nil,     8,    23,    34,     6,
   nil,   nil,     6,   nil,   nil,     6,   nil,   nil,   nil,     7,
     6,   nil,     6,     6,   nil,     7,   nil,   nil,     7,   nil,
     6,     7,   nil,   nil,   nil,   nil,     7,   nil,     7,     7,
   nil,   nil,   nil,   nil,   nil,   nil,     7 ]

racc_goto_pointer = [
   nil,    16,   nil,    10,    20,    77,   133,   149,   109,   nil,
   nil,    93,   nil,    13,   -52,    29,   -68,     2,   -68,     4,
     7,   -54,   -60,    -5,    -2,   -14,   nil,   nil,   -81,   -88,
   nil,   nil,   -50,   -75,    -4,   -93,   -41,  -114,    28,    23,
   nil,     9 ]

racc_goto_default = [
   nil,   nil,    17,    22,    69,    71,    53,    55,    59,    13,
    16,    66,    24,   nil,   nil,   nil,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,    91,   nil,    65,   105,    97,   107,
    89,   100,    90,    93,    95,   104,   106,   138,   nil,     4,
     8,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :EX_LOW => 2,
 :QUOTE => 3,
 :BAR => 4,
 :SLASH => 5,
 :BACK_SLASH => 6,
 :URL => 7,
 :OTHER => 8,
 :REF_OPEN => 9,
 :FOOTNOTE_OPEN => 10,
 :FOOTNOTE_CLOSE => 11,
 :EX_HIGH => 12,
 :EM_OPEN => 13,
 :EM_CLOSE => 14,
 :CODE_OPEN => 15,
 :CODE_CLOSE => 16,
 :VAR_OPEN => 17,
 :VAR_CLOSE => 18,
 :KBD_OPEN => 19,
 :KBD_CLOSE => 20,
 :INDEX_OPEN => 21,
 :INDEX_CLOSE => 22,
 :REF_CLOSE => 23,
 :VERB_OPEN => 24,
 :VERB_CLOSE => 25 }

racc_use_result_var = true

racc_nt_base = 26

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n,
 racc_use_result_var ]

Racc_debug_parser = false

##### racc system variables end #####


 # reduce 0 omitted

 # reduce 1 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 23 )
  def _reduce_2( val, _values, result )
 result.push(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 24 )
  def _reduce_3( val, _values, result )
 result = val
   result
  end
.,.,

 # reduce 4 omitted

 # reduce 5 omitted

 # reduce 6 omitted

 # reduce 7 omitted

 # reduce 8 omitted

 # reduce 9 omitted

 # reduce 10 omitted

 # reduce 11 omitted

 # reduce 12 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 38 )
  def _reduce_13( val, _values, result )
				result = Emphasis.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 41 )
  def _reduce_14( val, _values, result )
				result = Code.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 44 )
  def _reduce_15( val, _values, result )
				result = Var.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 47 )
  def _reduce_16( val, _values, result )
				result = Keyboard.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 50 )
  def _reduce_17( val, _values, result )
				result = Index.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 57 )
  def _reduce_18( val, _values, result )
 result = Reference.new(val[2], val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 59 )
  def _reduce_19( val, _values, result )
 result = Reference.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 62 )
  def _reduce_20( val, _values, result )
 result = Reference::URL.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 64 )
  def _reduce_21( val, _values, result )
 result = Reference::RDLabel.new(val[1], val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 66 )
  def _reduce_22( val, _values, result )
 result = Reference::RDLabel.new(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 67 )
  def _reduce_23( val, _values, result )
 result = Reference::RDLabel.new(nil, val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 69 )
  def _reduce_24( val, _values, result )
 result = Reference::URL.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 71 )
  def _reduce_25( val, _values, result )
 result = Reference::RDLabel.new(val[1], val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 73 )
  def _reduce_26( val, _values, result )
 result = Reference::RDLabel.new(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 74 )
  def _reduce_27( val, _values, result )
 result = Reference::RDLabel.new(nil, val[0])
   result
  end
.,.,

 # reduce 28 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 78 )
  def _reduce_29( val, _values, result )
 result = val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 80 )
  def _reduce_30( val, _values, result )
 result = [StringElement.new(val[1])]
   result
  end
.,.,

 # reduce 31 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 85 )
  def _reduce_32( val, _values, result )
 result = val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 90 )
  def _reduce_33( val, _values, result )
 result = [StringElement.new(val[0])]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 92 )
  def _reduce_34( val, _values, result )
 result = [StringElement.new(val[1])]
   result
  end
.,.,

 # reduce 35 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 98 )
  def _reduce_36( val, _values, result )
 result = val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 100 )
  def _reduce_37( val, _values, result )
 result = [StringElement.new(val[1])]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 104 )
  def _reduce_38( val, _values, result )
 result = val[1].unshift(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 106 )
  def _reduce_39( val, _values, result )
 result = val[1].unshift(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 108 )
  def _reduce_40( val, _values, result )
 result = val
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 109 )
  def _reduce_41( val, _values, result )
 result = val
   result
  end
.,.,

 # reduce 42 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 114 )
  def _reduce_43( val, _values, result )
 result.push(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 115 )
  def _reduce_44( val, _values, result )
 result = val
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 118 )
  def _reduce_45( val, _values, result )
 result.push(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 119 )
  def _reduce_46( val, _values, result )
 result = val
   result
  end
.,.,

 # reduce 47 omitted

 # reduce 48 omitted

 # reduce 49 omitted

 # reduce 50 omitted

 # reduce 51 omitted

 # reduce 52 omitted

 # reduce 53 omitted

 # reduce 54 omitted

 # reduce 55 omitted

 # reduce 56 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 136 )
  def _reduce_57( val, _values, result )
 result = StringElement.new(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 139 )
  def _reduce_58( val, _values, result )
 result = StringElement.new(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 142 )
  def _reduce_59( val, _values, result )
 result = StringElement.new(val[0])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 146 )
  def _reduce_60( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 61 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 152 )
  def _reduce_62( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 63 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 156 )
  def _reduce_64( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 65 omitted

 # reduce 66 omitted

 # reduce 67 omitted

 # reduce 68 omitted

 # reduce 69 omitted

 # reduce 70 omitted

 # reduce 71 omitted

 # reduce 72 omitted

 # reduce 73 omitted

 # reduce 74 omitted

 # reduce 75 omitted

 # reduce 76 omitted

 # reduce 77 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 179 )
  def _reduce_78( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 79 omitted

 # reduce 80 omitted

 # reduce 81 omitted

 # reduce 82 omitted

 # reduce 83 omitted

 # reduce 84 omitted

 # reduce 85 omitted

 # reduce 86 omitted

 # reduce 87 omitted

 # reduce 88 omitted

 # reduce 89 omitted

 # reduce 90 omitted

 # reduce 91 omitted

 # reduce 92 omitted

 # reduce 93 omitted

 # reduce 94 omitted

 # reduce 95 omitted

 # reduce 96 omitted

 # reduce 97 omitted

 # reduce 98 omitted

 # reduce 99 omitted

 # reduce 100 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 210 )
  def _reduce_101( val, _values, result )
				result = Footnote.new(val[1])
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 213 )
  def _reduce_102( val, _values, result )
				result = Verb.new(val[1])
   result
  end
.,.,

 # reduce 103 omitted

 # reduce 104 omitted

 # reduce 105 omitted

 # reduce 106 omitted

 # reduce 107 omitted

 # reduce 108 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 227 )
  def _reduce_109( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 110 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 232 )
  def _reduce_111( val, _values, result )
 result = StringElement.new(val[0])
   result
  end
.,.,

 # reduce 112 omitted

 # reduce 113 omitted

 # reduce 114 omitted

 # reduce 115 omitted

 # reduce 116 omitted

 # reduce 117 omitted

 # reduce 118 omitted

 # reduce 119 omitted

 # reduce 120 omitted

 # reduce 121 omitted

 # reduce 122 omitted

 # reduce 123 omitted

 # reduce 124 omitted

 # reduce 125 omitted

 # reduce 126 omitted

 # reduce 127 omitted

 # reduce 128 omitted

 # reduce 129 omitted

 # reduce 130 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 255 )
  def _reduce_131( val, _values, result )
 result = val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 256 )
  def _reduce_132( val, _values, result )
 result = val[1]
   result
  end
.,.,

 # reduce 133 omitted

 module_eval( <<'.,.,', 'rd/rdinlineparser.ry', 259 )
  def _reduce_134( val, _values, result )
 result << val[1]
   result
  end
.,.,

 # reduce 135 omitted

 def _reduce_none( val, _values, result )
  result
 end

end
end # end of module RD

